/*******************************************************************************
*
* E M B E D D E D   W I Z A R D   P R O J E C T
*
*                                                Copyright (c) TARA Systems GmbH
*                                    written by Paul Banach and Manfred Schweyer
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* TARA Systems hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   This file contains the necessary initialization of the ST7789V display
*   controller and the necessary function to update a rectangular area of the
*   screen from an Embedded Wizard UI application.
*   The data transfer is done by using the hardware GPIO controller via DMA.
*   The color format of the display is RGB565.
*
*******************************************************************************/

#include "ewconfig.h"
#include "cy_pdl.h"
#include "cyhal.h"
#include "cybsp.h"
#include "FreeRTOS.h"
#include "task.h"

#include "st7789v_i8080_gpio.h"



/* ST7789V commands */
#define ST7789V_SWRESET                 0x01   /* Software Reset */
#define ST7789V_RDDID                   0x04   /* Read Display ID Information */
#define ST7789V_RDDST                   0x09   /* Read Display Status */
#define ST7789V_RDDPM                   0x0A   /* Read Display Power Mode */
#define ST7789V_RDDMADCTL               0x0B   /* Read Display MADCTL */
#define ST7789V_RDDCOLMOD               0x0C   /* Read Display Pixel Format */
#define ST7789V_RDDIM                   0x0D   /* Read Display Image Format */
#define ST7789V_RDDSM                   0x0E   /* Read Display Signal Mode */
#define ST7789V_RDDSDR                  0x0F   /* Read Display Self-Diagnostic Result */
#define ST7789V_SPLIN                   0x10   /* Enter Sleep Mode */
#define ST7789V_SLPOUT                  0x11   /* Sleep out */
#define ST7789V_PTLON                   0x12   /* Partial Mode ON */
#define ST7789V_NORON                   0x13   /* Normal Display Mode ON */
#define ST7789V_INVOFF                  0x20   /* Display Inversion OFF */
#define ST7789V_INVON                   0x21   /* Display Inversion ON */
#define ST7789V_GAMSET                  0x26   /* Gamma register */
#define ST7789V_DISPOFF                 0x28   /* Display off */
#define ST7789V_DISPON                  0x29   /* Display on */
#define ST7789V_CASET                   0x2A   /* Colomn address */
#define ST7789V_RASET                   0x2B   /* Row address */
#define ST7789V_RAMWR                   0x2C   /* Memory Write */
#define ST7789V_RAMRD                   0x2E   /* Memory Read */
#define ST7789V_PLTAR                   0x30   /* Partial Area */
#define ST7789V_VSCRDEF                 0x33   /* Vertical Scrolling Definition */
#define ST7789V_TEOFF                   0x34   /* Tearing Effect Line OFF */
#define ST7789V_TEON                    0x35   /* Tearing Effect Line ON */
#define ST7789V_MADCTL                  0x36   /* Memory Access Control */
#define ST7789V_VSCSAD                  0x37   /* Vertical Scrolling Start Address */
#define ST7789V_IDMOFF                  0x38   /* Idle Mode OFF */
#define ST7789V_IDMON                   0x39   /* Idle Mode ON */
#define ST7789V_COLMOD                  0x3A   /* Pixel Format register */
#define ST7789V_WRMEMC                  0x3C   /* Write Memory Continue */
#define ST7789V_RDMEMC                  0x3E   /* Read Memory Continue */
#define ST7789V_STE                     0x44   /* Set Tear Scanline */
#define ST7789V_GSCAN                   0x45   /* Get Scanline */
#define ST7789V_WRDISBV                 0x51   /* Write Brightness Display */
#define ST7789V_RDDISBV                 0x52   /* Read Display Brightness */
#define ST7789V_WRCTRLD                 0x53   /* Write Control Display */
#define ST7789V_RDCTRLD                 0x54   /* Read CTRL Display */
#define ST7789V_WRCACE                  0x55   /* Write Content Adaptive Brightness Control */
#define ST7789V_RDCABC                  0x56   /* Read Content Adaptive Brightness Control */
#define ST7789V_WRCABCMB                0x5E   /* Write CABC Minimum Brightness */
#define ST7789V_RDCABCMB                0x5F   /* Read CABC Minimum Brightness */
#define ST7789V_RDABCSDR                0x68   /* Read Automatic Brightness Control Self Diagnostic Result */
#define ST7789V_READ_ID1                0xDA   /* Read ID1 */
#define ST7789V_READ_ID2                0xDB   /* Read ID2 */
#define ST7789V_READ_ID3                0xDC   /* Read ID3 */
#define ST7789V_RAMCTRL                 0xB0   /* RAM Control */
#define ST7789V_RGBCTRL                 0xB1   /* RGB Interface Control */
#define ST7789V_PORCTRL                 0xB2   /* Porch Setting */
#define ST7789V_FRCTRL1                 0xB3   /* Frame Rate Control (In Partial Mode) */
#define ST7789V_PARCTRL                 0xB5   /* Partial Mode Control */
#define ST7789V_GCTRL                   0xB7   /* Gate Control */
#define ST7789V_GTADJ                   0xB8   /* Gate On Timing Adjustment */
#define ST7789V_DGMEN                   0xBA   /* Digital Gamma Enable */
#define ST7789V_VCOMS                   0xBB   /* VCOMS Settings */
#define ST7789V_LCMCTRL                 0xC0   /* LCM Control */
#define ST7789V_IDSET                   0xC1   /* ID Code Setting */
#define ST7789V_VDVVRHEN                0xC2   /* VDV and VRH Command Enable */
#define ST7789V_VRHS                    0xC3   /* VRH Set */
#define ST7789V_VDVS                    0xC4   /* VDV Set */
#define ST7789V_VCMOFSET                0xC5   /* VCOMS Offset Set */
#define ST7789V_FRCTRL2                 0xC6   /* Frame Rate Control in Normal Mode */
#define ST7789V_CABCCTRL                0xC7   /* CABC Control */
#define ST7789V_REGSEL1                 0xC8   /* Register Value Selection 1 */
#define ST7789V_REGSEL2                 0xCA   /* Register Value Selection 2 */
#define ST7789V_PWMFRSEL                0xCC   /* PWM Frequency Selection */
#define ST7789V_PWCTRL1                 0xD0   /* Power Control 1 */
#define ST7789V_VAPVANEN                0xD2   /* Enable VAP/VAN singal output */
#define ST7789V_CMD2EN                  0xDF   /* Command 2 Enable */
#define ST7789V_PVGAMCTRL               0xE0   /* Positive Gamma Correction */
#define ST7789V_NVGAMCTRL               0xE1   /* Negative Gamma Correction */
#define ST7789V_DGMLUTR                 0xE2   /* Digital Gamma LUT Red */
#define ST7789V_DGMLUTB                 0xE3   /* Digital Gamma LUT Blue */
#define ST7789V_GATECTRL                0xE4   /* Gate Control */
#define ST7789V_SPI2EN                  0xE7   /* SPI2 Enable */
#define ST7789V_PWCTRL2                 0xE8   /* Power Control 2 */
#define ST7789V_EQCTRL                  0xE9   /* Equalize time Control */
#define ST7789V_PROMCTRL                0xEC   /* Program Mode Control */
#define ST7789V_PROMEN                  0xFA   /* Program Mode Enable */
#define ST7789V_NVMSET                  0xFC   /* NVM Setting */
#define ST7789V_PROMACT                 0xFE   /* Program Action */


/*******************************************************************************
* FUNCTION:
*   ST7789V_Init
*
* DESCRIPTION:
*   The function initializes the gpio hardware in order
*   to reset and intialize the connected display hardware.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void ST7789V_Init(void)
{
  /* Initialize the display controller */
  st7789v_i8080_init();


  /* Reset the display controller */
  st7789v_i8080_reset(0u);
  vTaskDelay(100);
  st7789v_i8080_reset(1u);
  vTaskDelay(50);

  /* Exit Sleep mode */
  st7789v_i8080_write_command(ST7789V_DISPOFF);
  st7789v_i8080_write_command(ST7789V_SLPOUT);

  vTaskDelay(100);

  /* MADCTL: memory data access control */
  st7789v_i8080_write_command(ST7789V_MADCTL);
  st7789v_i8080_write_data(0xA8);

  /* COLMOD: Interface Pixel format */
  st7789v_i8080_write_command(ST7789V_COLMOD);
  st7789v_i8080_write_data(0x65);

  /* PORCTRK: Porch setting */
  st7789v_i8080_write_command(ST7789V_PORCTRL);
  st7789v_i8080_write_data(0x0C);
  st7789v_i8080_write_data(0x0C);
  st7789v_i8080_write_data(0x00);
  st7789v_i8080_write_data(0x33);
  st7789v_i8080_write_data(0x33);

  /* GCTRL: Gate Control */
  st7789v_i8080_write_command(ST7789V_GCTRL);
  st7789v_i8080_write_data(0x35);

  /* VCOMS: VCOM setting */
  st7789v_i8080_write_command(ST7789V_VCOMS);
  st7789v_i8080_write_data(0x2B);

  /* LCMCTRL: LCM Control */
  st7789v_i8080_write_command(ST7789V_LCMCTRL);
  st7789v_i8080_write_data(0x2C);

  /* VDVVRHEN: VDV and VRH Command Enable */
  st7789v_i8080_write_command(ST7789V_VDVVRHEN);
  st7789v_i8080_write_data(0x01);
  st7789v_i8080_write_data(0xFF);

  /* VRHS: VRH Set */
  st7789v_i8080_write_command(ST7789V_VRHS);
  st7789v_i8080_write_data(0x11);

  /* VDVS: VDV Set */
  st7789v_i8080_write_command(ST7789V_VDVS);
  st7789v_i8080_write_data(0x20);

  /* FRCTRL2: Frame Rate control in normal mode */
  st7789v_i8080_write_command(ST7789V_FRCTRL2);
  st7789v_i8080_write_data(0x0F);

  /* PWCTRL1: Power Control 1 */
  st7789v_i8080_write_command(ST7789V_PWCTRL1);
  st7789v_i8080_write_data(0xA4);
  st7789v_i8080_write_data(0xA1);

  /* PVGAMCTRL: Positive Voltage Gamma control */
  st7789v_i8080_write_command(ST7789V_PVGAMCTRL);
  st7789v_i8080_write_data(0xD0);
  st7789v_i8080_write_data(0x00);
  st7789v_i8080_write_data(0x05);
  st7789v_i8080_write_data(0x0E);
  st7789v_i8080_write_data(0x15);
  st7789v_i8080_write_data(0x0D);
  st7789v_i8080_write_data(0x37);
  st7789v_i8080_write_data(0x43);
  st7789v_i8080_write_data(0x47);
  st7789v_i8080_write_data(0x09);
  st7789v_i8080_write_data(0x15);
  st7789v_i8080_write_data(0x12);
  st7789v_i8080_write_data(0x16);
  st7789v_i8080_write_data(0x19);

  /* NVGAMCTRL: Negative Voltage Gamma control */
  st7789v_i8080_write_command(ST7789V_NVGAMCTRL);
  st7789v_i8080_write_data(0xD0);
  st7789v_i8080_write_data(0x00);
  st7789v_i8080_write_data(0x05);
  st7789v_i8080_write_data(0x0D);
  st7789v_i8080_write_data(0x0C);
  st7789v_i8080_write_data(0x06);
  st7789v_i8080_write_data(0x2D);
  st7789v_i8080_write_data(0x44);
  st7789v_i8080_write_data(0x40);
  st7789v_i8080_write_data(0x0E);
  st7789v_i8080_write_data(0x1C);
  st7789v_i8080_write_data(0x18);
  st7789v_i8080_write_data(0x16);
  st7789v_i8080_write_data(0x19);

  /* Y address set */
  st7789v_i8080_write_command(ST7789V_RASET);
  st7789v_i8080_write_data(0x00);
  st7789v_i8080_write_data(0x00);
  st7789v_i8080_write_data(0x00);
  st7789v_i8080_write_data(0xEF);

  /* X address set */
  st7789v_i8080_write_command(ST7789V_CASET);
  st7789v_i8080_write_data(0x00);
  st7789v_i8080_write_data(0x00);
  st7789v_i8080_write_data(0x01);
  st7789v_i8080_write_data(0x3F);

  vTaskDelay(10);

  /* Enable display */
  st7789v_i8080_write_command(ST7789V_DISPON);
}


/*******************************************************************************
* FUNCTION:
*   ST7789V_WriteDataWindow
*
* DESCRIPTION:
*   The function sets the destination position and size within the framebuffer
*   of the display according the given rectangle and starts the transfer of
*   the given data. The function does NOT wait for the completion of the transfer.
*
* ARGUMENTS:
*   aX       - Horizontal position for next write access.
*   aY       - Vertical position for next write access.
*   aW       - Width of the data window in pixel.
*   aH       - Height of the data window in pixel.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void ST7789V_WriteDataWindow( uint16_t aX, uint16_t aY, uint16_t aW, uint16_t aH, uint8_t* aData )
{
  int       xEnd = aX + aW - 1;
  int       yEnd = aY + aH - 1;

  /* X address set */
  st7789v_i8080_write_command(ST7789V_CASET);
  st7789v_i8080_write_data(aX >> 8);
  st7789v_i8080_write_data(aX & 0xFF);
  st7789v_i8080_write_data(xEnd >> 8);
  st7789v_i8080_write_data(xEnd & 0xFF);

  /* Y address set */
  st7789v_i8080_write_command(ST7789V_RASET);
  st7789v_i8080_write_data(aY >> 8);
  st7789v_i8080_write_data(aY & 0xFF);
  st7789v_i8080_write_data(yEnd >> 8);
  st7789v_i8080_write_data(yEnd & 0xFF);

  st7789v_i8080_write_command(ST7789V_RAMWR);
  st7789v_i8080_write_data_stream(aData, aW * aH * 2);
}

/* mli */












