/*******************************************************************************
*
* E M B E D D E D   W I Z A R D   P R O J E C T
*
*                                                Copyright (c) TARA Systems GmbH
*                                    written by Paul Banach and Manfred Schweyer
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* TARA Systems hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   This file contains the necessary initialization of the ST7789V display
*   controller and the necessary function to update a rectangular area of the
*   screen from an Embedded Wizard UI application.
*   The data transfer is done by using the hardware GPIO controller via DMA.
*   The color format of the display is RGB565.
*
*******************************************************************************/

#include "ewconfig.h"
#include "cy_pdl.h"
#include "cyhal.h"
#include "cybsp.h"



#if EW_USE_OPTIMIZED_LCD_PINNING == 1

/*******************************************************************************
 * Display wiring modifications
 *
 * If using EW_USE_OPTIMIZED_LCD_PINNING option, following display wirings have
 * to be changed.
 *
 * Display pin    original                    optimized
 * ----------------------------------------------------
 * DB11           J2, A12              ->     J2, A11
 * DB12           J2, A13              ->     J2, A12
 * DB13           J4, D7               ->     J2, A13
 * DB14           J3, D8               ->     J2, A14
 * DB15           J3, D9               ->     J2, A15
 *******************************************************************************/

  #define I8080_PIN_D0    P9_0
  #define I8080_PIN_D1    P9_1
  #define I8080_PIN_D2    P9_2
  #define I8080_PIN_D3    P9_3
  #define I8080_PIN_D4    P9_4
  #define I8080_PIN_D5    P9_5
  #define I8080_PIN_D6    P9_6
  #define I8080_PIN_D7    P9_7

#else

  #define I8080_PIN_D0    CYBSP_J2_2   // P9_0
  #define I8080_PIN_D1    CYBSP_J2_4   // P9_1
  #define I8080_PIN_D2    CYBSP_J2_6   // P9_2
  #define I8080_PIN_D3    CYBSP_J2_10  // P9_4
  #define I8080_PIN_D4    CYBSP_J2_12  // P9_5
  #define I8080_PIN_D5    CYBSP_D7     // P5_7
  #define I8080_PIN_D6    CYBSP_D8     // P7_5
  #define I8080_PIN_D7    CYBSP_D9     // P7_6


#endif

#define I8080_PIN_RD      CYBSP_D10  // P12_3
#define I8080_PIN_WR      CYBSP_D11  // P12_0
#define I8080_PIN_DC      CYBSP_D12  // P12_1
#define I8080_PIN_RS      CYBSP_D13  // P12_2

#define I8080_DATA_PORT   GPIO_PRT9

#define GET_PORTADDR(pin) ((GPIO_PRT_Type *)(GPIO_BASE + (GPIO_PRT_SECTION_SIZE * CYHAL_GET_PORT(pin))))
#define CLR_PIN( pin )    GET_PORTADDR( pin )->OUT_CLR = 1 << CYHAL_GET_PIN( pin )
#define SET_PIN( pin )    GET_PORTADDR( pin )->OUT_SET = 1 << CYHAL_GET_PIN( pin )


/*******************************************************************************
 * Writes one byte of data to the i8080 interface.
 *******************************************************************************/
static inline void st7789v_i8080_write(uint8_t data)
{
#if EW_USE_OPTIMIZED_LCD_PINNING == 1

	I8080_DATA_PORT->OUT = data;

#else

	/* platform specific implementation for CY8CKIT-062S2-43012 and CY8CKIT-028-TFT */
    GPIO_PRT9->OUT = ( GPIO_PRT9->OUT & 0b11001000) | (( data & 0b00000111 ) >> 0 )
                                                    | (( data & 0b00011000 ) << 1 );
    GPIO_PRT7->OUT = ( GPIO_PRT9->OUT & 0b10011111) | (( data & 0b11000000 ) >> 1 );
    GPIO_PRT5->OUT = ( GPIO_PRT9->OUT & 0b01111111) | (( data & 0b00100000 ) << 2 );

#endif

    CLR_PIN( I8080_PIN_WR );
    SET_PIN( I8080_PIN_WR );
}


/*******************************************************************************
 * Initializes GPIOs for the i8080 8-bit interface.
 *******************************************************************************/
void st7789v_i8080_init( void )
{
  cyhal_gpio_init( I8080_PIN_D0, CYHAL_GPIO_DIR_OUTPUT, CYHAL_GPIO_DRIVE_STRONG, 0 );
  cyhal_gpio_init( I8080_PIN_D1, CYHAL_GPIO_DIR_OUTPUT, CYHAL_GPIO_DRIVE_STRONG, 0 );
  cyhal_gpio_init( I8080_PIN_D2, CYHAL_GPIO_DIR_OUTPUT, CYHAL_GPIO_DRIVE_STRONG, 0 );
  cyhal_gpio_init( I8080_PIN_D3, CYHAL_GPIO_DIR_OUTPUT, CYHAL_GPIO_DRIVE_STRONG, 0 );
  cyhal_gpio_init( I8080_PIN_D4, CYHAL_GPIO_DIR_OUTPUT, CYHAL_GPIO_DRIVE_STRONG, 0 );
  cyhal_gpio_init( I8080_PIN_D5, CYHAL_GPIO_DIR_OUTPUT, CYHAL_GPIO_DRIVE_STRONG, 0 );
  cyhal_gpio_init( I8080_PIN_D6, CYHAL_GPIO_DIR_OUTPUT, CYHAL_GPIO_DRIVE_STRONG, 0 );
  cyhal_gpio_init( I8080_PIN_D7, CYHAL_GPIO_DIR_OUTPUT, CYHAL_GPIO_DRIVE_STRONG, 0 );

  cyhal_gpio_init( I8080_PIN_RD, CYHAL_GPIO_DIR_OUTPUT, CYHAL_GPIO_DRIVE_STRONG, 1 );
  cyhal_gpio_init( I8080_PIN_WR, CYHAL_GPIO_DIR_OUTPUT, CYHAL_GPIO_DRIVE_STRONG, 1 );
  cyhal_gpio_init( I8080_PIN_DC, CYHAL_GPIO_DIR_OUTPUT, CYHAL_GPIO_DRIVE_STRONG, 0 );
  cyhal_gpio_init( I8080_PIN_RS, CYHAL_GPIO_DIR_OUTPUT, CYHAL_GPIO_DRIVE_STRONG, 1 );
}


/*******************************************************************************
 * Free all resources used for the software i8080 interface.
 *******************************************************************************/
void st7789v_i8080_done(void)
{
    cyhal_gpio_free( I8080_PIN_D0 );
    cyhal_gpio_free( I8080_PIN_D1 );
    cyhal_gpio_free( I8080_PIN_D2 );
    cyhal_gpio_free( I8080_PIN_D3 );
    cyhal_gpio_free( I8080_PIN_D4 );
    cyhal_gpio_free( I8080_PIN_D5 );
    cyhal_gpio_free( I8080_PIN_D6 );
    cyhal_gpio_free( I8080_PIN_D7 );
    cyhal_gpio_free( I8080_PIN_RD );
    cyhal_gpio_free( I8080_PIN_WR );
    cyhal_gpio_free( I8080_PIN_DC );
    cyhal_gpio_free( I8080_PIN_RS );
}


/*******************************************************************************
 * Sets value of the display Reset pin.
 *******************************************************************************/
void st7789v_i8080_reset( bool value )
{
  if ( value )
    SET_PIN( I8080_PIN_RS );
  else
    CLR_PIN( I8080_PIN_RS );
}


/*******************************************************************************
 * Writes one byte of data to the software i8080 interface with the LCD_DC pin
 *******************************************************************************/
void st7789v_i8080_write_command(uint8_t data)
{
  CLR_PIN( I8080_PIN_DC );
  st7789v_i8080_write(data);
}


/*******************************************************************************
 * Writes one byte of data to the software i8080 interface with the LCD_DC pin
 *******************************************************************************/
void st7789v_i8080_write_data(uint8_t data)
{
  SET_PIN( I8080_PIN_DC );
  st7789v_i8080_write(data);
}


/*******************************************************************************
 * Writes multiple bytes of data to the software i8080 interface with the LCD_DC
 * pin set to 1.
 *******************************************************************************/
void st7789v_i8080_write_data_stream(uint8_t *data, int num)
{
  int i;

  SET_PIN( CYBSP_D12 );

  for (i = 0; i < num; i++)
  {
	st7789v_i8080_write(data[i]);
  }
}
